/*
	Template Instructions:
	
	Search for "Rectangle", and edit as needed with consideration of your class name
	
	Search for "//TODO" tags. Each tag has a note.	
*/



/*
	controller.js
	Controller is interface and event manager between View and model
*/

/*
	If using this file as a template for another unit test:
	
		1) There are a few "//framework" tags in this file. A 
		minor edit or no edit is needed (comment by tag will inform)
		2) The expression "extends TestCase" below is also
		part of the framework support (no edit required)
*/

class Controller extends AbstractController {
	
	//----------------------------
	//Virtual - required
	
	newModel() {
		//framework -- name of model class
		return new Rectangle(0, 0);
	}
	
	//----------------------------
	//Virtual - optional
	
	getRefreshableFields() {
		//TODO
		//framework -- add any field names here that are output/readonly.
		//Model should have getter for each, e.g. getWidth, getHeight, etc.	
		return ['width', 'height', 'area', 'perimeter', 'diagonal', 'message', 'tip', 'log'];
	}		
	
	//----------------------------
	//Attach (connect) element events to handler methods
	
	attachEvents() {
		//TODO
		//connect events from your fields and buttons (from view) 
		//to controller event handler methods (which you will code)
		//examples shown here
	
		//Attach field event to event handler
		//ui element name, event name, event handler method name
		super.attachEvents();
		
		//actions
		this.connectEvent('flip', 'click', 'clickedFlip');				
		this.connectEvent('multiply', 'click', 'clickedMultiply');
		
		//"change" events (e.g. when a value is typed into a ui input)
		this.connectEvent('width', 'change', 'changedWidth');		
		this.connectEvent('height', 'change', 'changedHeight');				
	}
	
	//-------------------------------
	//Change event handlers -- e.g. when we type something into a field
	
	//TODO
	//Change "change event" handler names and logic to match your handlers
	//set model value from view
	//Use "parseFloat" if model setter is expecting number
	
	changedWidth(field) {
		//set the view value from ui element "field" into model (convert to number)
		this.model.setWidth(parseFloat(field.value));
	}
	
	//-------------------------------
	//Action event handlers -- actions (e.g. button clicks 'clicked')
	
	//TODO
	//Change "click event" handler names and logic to match your handlers
	//We don't want these methods too long (try to keep core logic in model)
	
	clickedFlip() {
		this.model.flip();
	}	
	
	clickedMultiply() {
		//Get factor from user
		const factorString = prompt("Please enter factor to multiply rectangle dimensions by", "2");	

		//If canceled, return
		if (factorString == null)
			return;
	
		//convert number to string using helper "coerceNum"
		const factor = coerceNum(factorString);
		this.model.multiply(factor);		
	}		
	
	//-------------------------------
	//User friendly type mvc logic here (optional)

	//TODO
	//Optional, if you want to do any cosmetics,  e.g. background color, etc.
	//this is a good place
	//Note that "this.elem('flip')" gets the ui element (e.g. button or field)
		
	refreshState() {
		//framework
		//This gives us an opportunity to enable/disable a button per state, etc.
		//OPTIONAL 
		super.refreshState();
		const err = this.model.hasError();
		//if error, then disable buttons
		this.elem('flip').disabled = err;
		this.elem('multiply').disabled = err;		
		//If model has error, then set background of "message" to yellow (null means default)
		this.elem('message').style.backgroundColor = err ? 'yellow' : null;
	}
	
}

//======================================================
//No changes required

function buildController() {
	//one and only controller (global)
	controller = new Controller();
}	


