'use strict';
/*
	File: model-test-case.js
	Class: RollTheDiceTestCase
*/

/*
	If using this file as a template for another unit test:
	
		1) There are a few "//framework" tags in this file. A 
		minor edit or no edit is needed (comment by tag will inform)
		2) The expression "extends TestCase" below is also
		part of the framework support (no edit required)
*/


//framework - argument here should be name of this test case
addTestCase("RollTheDiceTestCase");

class RollTheDiceTestCase extends TestCase {
	//ivars: none
	
	//-----------------
	//Unit Test Support		
	
	getUnitName() {
		//framework - return name of model class here
		return 'RollTheDice';
	}	
	
	//-----------------
	//Setup

	beforeEach() {
		//Called before each test for test setup (optional method)
		super.beforeEach();
		//Construct a unit model for testing
		//framework - name of model class here
		this.unit = new RollTheDice();
	}		
		
	//-----------------
	//Tests
	
	testConstruction() {
		//Initially both player and computer should have 0 points
		const game = this.unit;
		this.assertEquals(0, game.getPlayerPoints());
		this.assertEquals(0, game.getComputerPoints());		
	}
	
	testVeryLuckyPlayer() {
		//player rolls lucky twos twice in a row to win
		//test that "message" includes "winner" and "player"
		const game = this.unit;
		game.processPlayerRoll(2, 2);
		game.processPlayerRoll(2, 2);
		//expected result, actual result, info message
		//third arg -- getMessage() -- is cosmetic info message (if assert fails)		
		this.assertEquals(true, game.isPlayerWinner(), game.getMessage());
	}	
	
	testVeryLuckyComputer() {
		//computer rolls lucky twos twice in a row to win
		//test that "message" includes "winner" and "computer"
		const game = this.unit;
		game.processComputerRoll(2, 2);
		game.processComputerRoll(2, 2);
		this.assertEquals(true, game.isComputerWinner(), game.getMessage());		
	}		
	
	testCloseGame() {
		//test simulation of player winning in close game
		const game = this.unit;
		game.processPlayerRoll(6, 5);		//11
		game.processComputerRoll(4, 6);		//10
		game.processPlayerRoll(2, 2);		//11+15=26
		game.processComputerRoll(2, 2);		//10+15=25		
		game.processPlayerRoll(1, 3);		//26+4=30 WINNER
		this.assertEquals(true, game.isPlayerWinner(), game.getMessage());		
	}		
	
	testNoWinner() {
		//test simulation of game at point where there is not yet a winner
		const game = this.unit;
		game.processPlayerRoll(6, 5);		//11
		game.processComputerRoll(4, 6);		//10
		this.assertEquals(false, game.hasWinner(), game.getMessage());		
	}	
	
	testBadPlayerLuck() {
		//simulate player rolling up to 27 but then hits bad luck
		const game = this.unit;
		game.processPlayerRoll(2, 2);		//15
		game.processComputerRoll(6, 6);		//12
		game.processPlayerRoll(6, 6);		//27
		game.processComputerRoll(6, 6);		//24		
		game.processPlayerRoll(1, 1);		//0
		game.processComputerRoll(6, 6);		//36 (WINNER)
		this.assertEquals(true, game.isComputerWinner(), game.getMessage());
	}
	
	testPublicProtocol() {
		//We have tested private methods e.g. "processPlayerRoll"
		//Let's make sure the public method runs. It is random, so we
		//will not check the score, but just a sanity check
		const game = this.unit;
		game.playerRoll();
		game.computerRoll();
		//It's random, but we do know the score should be non-zero
		this.assertEquals(true, game.getPlayerPoints() !== 0, 'asserting player score is non-zero');		
		this.assertEquals(true, game.getComputerPoints() !== 0, 'asserting computer score is non-zero');		
	}		


	
} 
	
