/*
	File: model.js
	Class: Rectangle	
	
	This Rectangle shows some examples of messages related
	to logic. They can be used as examples as we might
	build a game.
	
	NOTE WELL
		Several ways you can "run" this code.
		1. There is a test case ready to go (see "test" directory)
		2. There is a view ready to go (see "ui" directory)
		3. At the bottom of file there is a small script.
		So if you have a Node.js editor (e.g. Notepad++/other)
		setup, you could just run this file.
		
	//framework
	1) Code tagged with the tag //framework requires
	a minor edit or no edit (comment by tag will inform)
	2) The expression "extends superClass" below is also
	part of the framework support (no edit required)
*/

//framework - no edit needed
let superClass = class {};
try { superClass = Model; } catch(ex) {}


class Rectangle extends superClass {
	/*ivars
	width
	height
	log*/
	
	//------------------------------------------------	
		
	constructor(w, h) {
		super();
		//
		if (w === undefined || h === undefined)
			throw new Error("Must construct a Rectangle with two params");		
		this.width = w;
		this.height = h;
		this.log = '';
	}
	
	//------------------------------------------------	
	//Getters (Basic)
	
	getWidth() {
		return this.width;
	}
	
	getHeight() {
		return this.height;
	}
	
	getLog() {
		return this.log;
	}	
	
	//------------------------------------------------
	//Setters
	
	setWidth(value) {
		this.log = `Thank you for setting width to ${value}`;
		this.width = value;	
	}
	
	setHeight(value) {
		this.log = `Thank you for setting height to ${value}`;		
		this.height = value;				
	}		
	
	//------------------------------------------------	
	//Derived Getters
	
	getArea() {
		return this.width * this.height;
	}	
	
	getPerimeter() {
		return 2 * (this.width + this.height);
	}
	
	getDiagonal() {
		const
			w = this.width,
			h = this.height;
		return Math.sqrt(w*w + h*h);
	}
	
	getError() {
		if (this.width <= 0 || this.height <= 0)
			return 'ERROR: Dimensions must be greater than zero';
		return null;
	}
	
	hasError() {
		return this.getError() !== null;
	}

	getMessage() {
		if (this.hasError()) return this.getError();
		const 
			w = this.getWidth(), 
			h = this.getHeight();
		if (!w || !h) return 'Uninitialized';
		if (w === h) return 'Square';
		return (w > h) ? 'Wide' : 'Tall';
	}	
	
	getTip() {
		return 'Enter width and height';
	}	
		
	
	//------------------------------------------------	
	//Actions
	
	flip() {
		//Flip width and height
		const w = this.width;
		this.width = this.height;
		this.height = w;
	}		
	
	multiply(factor) {
		//xdlm
		this.width = this.width * factor;
		this.height = this.height * factor;		
	}	
	
	//------------------------------------------------	
	//Common
	
	toString() {
		return '' + this.getWidth() + 'x' + this.getHeight()
			+ ' area=' + this.getArea();
	}

}



//--------------------------------
//Playing

function prn(o) { console.log(o); }

const rec = new Rectangle(10, 5);
rec.setWidth(8);
prn(rec.getTip());
prn(rec.getLog());
prn(rec.getMessage());
//Force an error
rec.setWidth(-100);
prn(rec.getTip());
prn(rec.getLog());
prn(rec.getMessage());
