/*
	File: model-test-case.js
	Class: RectangleTestCase	
	
	//framework
	1) Code tagged with the tag //framework requires
	a minor edit or no edit (comment by tag will inform)
	2) The expression "extends superClass" below is also
	part of the framework support (no edit required)
*/

//framework -- test case name
addTestCase("RectangleTestCase");

class RectangleTestCase extends TestCase {
	//ivars: unit
	
	//-----------------
	//Unit Test Support		
	
	getUnitName() {
		//framework -- model type (class)
		return 'Rectangle';
	}	
	
	//-----------------
	//Setup	
	
	beforeEach() {
		//Called before each test for test setup (if needed)
		//Construct a unit model for testing (we use `lk`cf.ivar "unit")
		super.beforeEach();		
		//framework - name of model class here (and using the constructor that is available)		
		this.unit = new Rectangle(10, 5);
	}	
	
	//-----------------
	//Tests
	
	testGetWidth() {
		//Test that the unit (testee) returns width as expected (10) for `bq`"getWidth"
		this.assertEquals(10, this.unit.getWidth());
	}
	
	testGetHeight() {
		//Assert that the unit returns height as expected (5) for `bq`"getHeight"*/	
		this.assertEquals(5, this.unit.getHeight());
	}	
	
	testGetArea() {
		//Assert that the unit returns an area as expected (10*5) for `bq`"getArea"*/		
		this.assertEquals(10 * 5, this.unit.getArea());
	}			
	
	testGetPerimeter() {
		//Assert that testee returns a perimeter as expected (10+10+5+5) for `bq`"getPerimeter" */		
		this.assertEquals(10 + 10 + 5 + 5, this.unit.getPerimeter());
	}				
	
	testGetDiagonal() {
		//Floating point (float) comparisons are not exact comparisons. 
		//Thus we use a "tolerance" here of 0.0001 to allow a little "fuzz".
		this.assertFloatEquals(11.18034, this.unit.getDiagonal(), 0.0001);
	}			
	
	testFlip() {
		//Flip the unit and then assert that dimensions have flipped */			
		const rec = this.unit;
		rec.flip();
		this.assertEquals(5, this.unit.getWidth());
		this.assertEquals(10, this.unit.getHeight());		
	}	
	
	testMultiply() {
		//Multiply the unit dimensions and then assert the dimensions match expected values */				
		const rec = this.unit;
		rec.multiply(2);
		this.assertEquals(10 * 2, this.unit.getWidth());
		this.assertEquals(5 * 2, this.unit.getHeight());		
	}	
	
	testMessage1() {
		const rec = this.unit;
		//Force an error
		rec.setWidth(-100);
		//we expect word "error" in message with negative width
		this.assertEquals(true, rec.getMessage().toLowerCase().includes('error'));
	}		
	
	testMessage2() {
		//we expect NO "error" (e.g. false) for model constructed above
		this.assertEquals(false, this.unit.getMessage().toLowerCase().includes('error'));
	}		
	
	testMessage2() {
		//we expect NO "error" (e.g. false) for model constructed above
		this.assertEquals(false, this.unit.getMessage().toLowerCase().includes('error'));
	}	
	
	testLog() {
		//expecing word "width" in log after setting width
		this.assertEquals(false, this.unit.getLog().toLowerCase().includes('width'));
	}		
	
} 

